// DDutils.cpp
// Upraveno - oroginal od Microsoftu

#include "stdafx.h"
#include "ddutils.h" // nadstavba DirectDraw

/*
 *  DDLoadBitmap
 *
 *  create a DirectDrawSurface from a bitmap resource.
 *
 */
IDirectDrawSurface * DDLoadBitmap(IDirectDraw *pdd, LPCSTR szBitmap, int dx, int dy, HINSTANCE module)
{
    HBITMAP             hbm;
    BITMAP              bm;
    DDSURFACEDESC       ddsd;
    IDirectDrawSurface *pdds;

    //
    //  try to load the bitmap as a resource, if that fails, try it as a file
    //
    hbm = (HBITMAP)LoadImage(module, szBitmap, IMAGE_BITMAP, dx, dy, LR_CREATEDIBSECTION);

    if (hbm == NULL)
	hbm = (HBITMAP)LoadImage(NULL, szBitmap, IMAGE_BITMAP, dx, dy, LR_LOADFROMFILE|LR_CREATEDIBSECTION);

    if (hbm == NULL)
	return NULL;

    //
    // get size of the bitmap
    //
    GetObject(hbm, sizeof(bm), &bm);      // get size of bitmap

    //
    // create a DirectDrawSurface for this bitmap
    //
    ZeroMemory(&ddsd, sizeof(ddsd));
    ddsd.dwSize = sizeof(ddsd);
    ddsd.dwFlags = DDSD_CAPS | DDSD_HEIGHT |DDSD_WIDTH;
    ddsd.ddsCaps.dwCaps = DDSCAPS_OFFSCREENPLAIN;
    ddsd.dwWidth = bm.bmWidth;
    ddsd.dwHeight = bm.bmHeight;

    if (pdd->CreateSurface(&ddsd, &pdds, NULL) != DD_OK)
	return NULL;
	
    DDCopyBitmap(pdds, hbm, 0, 0, 0, 0);

    DeleteObject(hbm);

    return pdds;
}

/*
 *  DDReLoadBitmap
 *
 *  load a bitmap from a file or resource into a directdraw surface.
 *  normaly used to re-load a surface after a restore.
 *
 */
HRESULT DDReLoadBitmap(IDirectDrawSurface *pdds, LPCSTR szBitmap, HINSTANCE module)
{
    HBITMAP             hbm;
    HRESULT             hr;

    //
    //  try to load the bitmap as a resource, if that fails, try it as a file
    //
    hbm = (HBITMAP)LoadImage(module, szBitmap, IMAGE_BITMAP, 0, 0, LR_CREATEDIBSECTION);

    if (hbm == NULL)
	hbm = (HBITMAP)LoadImage(NULL, szBitmap, IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE|LR_CREATEDIBSECTION);

    if (hbm == NULL)
    {
	OutputDebugString("handle is null\n");
	return E_FAIL;
    }

    hr = DDCopyBitmap(pdds, hbm, 0, 0, 0, 0);
    if (hr != DD_OK)
    {
//	OutputDebugString("ddcopybitmap failed\n");
    }


    DeleteObject(hbm);
    return hr;
}


/*
 * DDColorMatch
 *
 * convert a RGB color to a pysical color.
 *
 * we do this by leting GDI SetPixel() do the color matching
 * then we lock the memory and see what it got mapped to.
 */
DWORD DDColorMatch(IDirectDrawSurface *pdds, COLORREF rgb)
{
    COLORREF rgbT;
    HDC hdc;
    DWORD dw = CLR_INVALID;
    DDSURFACEDESC ddsd;
    HRESULT hres;

    //
    //  use GDI SetPixel to color match for us
    //
    if (rgb != CLR_INVALID && pdds->GetDC(&hdc) == DD_OK)
    {
	rgbT = GetPixel(hdc, 0, 0);             // save current pixel value
	SetPixel(hdc, 0, 0, rgb);               // set our value
	pdds->ReleaseDC(hdc);
    }

    //
    // now lock the surface so we can read back the converted color
    //
    ddsd.dwSize = sizeof(ddsd);
    while ((hres = pdds->Lock(NULL, &ddsd, 0, NULL)) == DDERR_WASSTILLDRAWING)
	;

    if (hres == DD_OK)
    {
	dw  = *(DWORD *)ddsd.lpSurface;                     // get DWORD
        if(ddsd.ddpfPixelFormat.dwRGBBitCount < 32)
            dw &= (1 << ddsd.ddpfPixelFormat.dwRGBBitCount)-1;  // mask it to bpp
	pdds->Unlock(NULL);
    }

    //
    //  now put the color that was there back.
    //
    if (rgb != CLR_INVALID && pdds->GetDC(&hdc) == DD_OK)
    {
	SetPixel(hdc, 0, 0, rgbT);
	pdds->ReleaseDC(hdc);
    }

    return dw;
}

/*
 * DDSetColorKey
 *
 * set a color key for a surface, given a RGB.
 * if you pass CLR_INVALID as the color key, the pixel
 * in the upper-left corner will be used.
 */
HRESULT DDSetColorKey(IDirectDrawSurface *pdds, COLORREF rgb)
{
    DDCOLORKEY          ddck;

    ddck.dwColorSpaceLowValue  = DDColorMatch(pdds, rgb);
    ddck.dwColorSpaceHighValue = ddck.dwColorSpaceLowValue;
    pdds->SetColorKey(DDCKEY_DESTBLT, &ddck);
	 return pdds->SetColorKey(DDCKEY_SRCBLT, &ddck);
}

/*
 *  DDCopyBitmap
 *
 *  draw a bitmap into a DirectDrawSurface
 *
 */
HRESULT DDCopyBitmap(IDirectDrawSurface *pdds, HBITMAP hbm, int x, int y, int dx, int dy)
{
    HDC                 hdcImage;
    HDC                 hdc;
    BITMAP              bm;
    DDSURFACEDESC       ddsd;
    HRESULT             hr;

    if (hbm == NULL || pdds == NULL)
	return E_FAIL;

    //
    // make sure this surface is restored.
    //
    pdds->Restore();

    //
    //  select bitmap into a memoryDC so we can use it.
    //
    hdcImage = CreateCompatibleDC(NULL);
    if (!hdcImage)
	OutputDebugString("createcompatible dc failed\n");
    SelectObject(hdcImage, hbm);

    //
    // get size of the bitmap
    //
    GetObject(hbm, sizeof(bm), &bm);    // get size of bitmap
    dx = dx == 0 ? bm.bmWidth  : dx;    // use the passed size, unless zero
    dy = dy == 0 ? bm.bmHeight : dy;

    //
    // get size of surface.
    //
    ddsd.dwSize = sizeof(ddsd);
    ddsd.dwFlags = DDSD_HEIGHT | DDSD_WIDTH;
    pdds->GetSurfaceDesc(&ddsd);

    if ((hr = pdds->GetDC(&hdc)) == DD_OK)
    {
	StretchBlt(hdc, 0, 0, ddsd.dwWidth, ddsd.dwHeight, hdcImage, x, y, dx, dy, SRCCOPY);
	pdds->ReleaseDC(hdc);
    }

    DeleteDC(hdcImage);

    return hr;
}


/*
 * DDInitStruct
 *
 * reset given structure
 */
void DDInitStruct(DDSURFACEDESC *structure){
	ZeroMemory(structure, sizeof(DDSURFACEDESC));
	structure->dwSize                 = sizeof(DDSURFACEDESC);
	structure->ddpfPixelFormat.dwSize = sizeof(DDSURFACEDESC);
}

void DDInitStruct(DDPIXELFORMAT *structure){
	ZeroMemory(structure, sizeof(DDPIXELFORMAT));
	structure->dwSize  = sizeof(DDPIXELFORMAT);
}

void DDInitStruct(DDBLTFX *structure){
	ZeroMemory(structure, sizeof(DDBLTFX));
	structure->dwSize  = sizeof(DDBLTFX);
}


/*
// the structure which holds pixel RGB masks
typedef struct _RGBMASK
{
   unsigned long rgbRed; // red component
   unsigned long rgbGreen; // green component
   unsigned long rgbBlue; // blue component
} RGBMASK;

// the structure which holds screen surface info (5,6,5 or 5,5,5 and masking)
typedef struct _RGB16
{
   RGBQUAD depth;
   RGBQUAD Amount;
   RGBQUAD Position;
   RGBMASK Mask;
} RGB16;

RGB16 rgb16;

// Must run this proc to fill the RGB16 struct with the information needed to plot a pixel
// To call this, you must have rgb16 defined as a global (unless you want to modify this) variable
// RGB16 rgb16;
BOOL GetRGB16(LPDIRECTDRAWSURFACE Surface)
{
   DDSURFACEDESC   ddsd;
   BYTE shiftcount;
   
   // get a surface despriction
   ddsd.dwSize = sizeof(ddsd);
   ddsd.dwFlags = DDSD_PIXELFORMAT;
   if(Surface->GetSurfaceDesc(&ddsd) != DD_OK)
      return FALSE;

   // Fill in the masking values for extracting colors
   rgb16.Mask.rgbRed = ddsd.ddpfPixelFormat.dwRBitMask;
   rgb16.Mask.rgbGreen = ddsd.ddpfPixelFormat.dwGBitMask;
   rgb16.Mask.rgbBlue = ddsd.ddpfPixelFormat.dwBBitMask;

   // get red
   shiftcount = 0;
   while(!(ddsd.ddpfPixelFormat.dwRBitMask & 1))
   {
      ddsd.ddpfPixelFormat.dwRBitMask >>= 1;
      shiftcount++;
   }
   rgb16.depth.rgbRed = (BYTE)ddsd.ddpfPixelFormat.dwRBitMask;
   rgb16.Position.rgbRed = shiftcount;
   rgb16.Amount.rgbRed = (ddsd.ddpfPixelFormat.dwRBitMask == 0x1f) ? 3 : 2;

   // get green
   shiftcount = 0;
   while(!(ddsd.ddpfPixelFormat.dwGBitMask & 1))
   {
      ddsd.ddpfPixelFormat.dwGBitMask >>= 1;
      shiftcount++;
   }
   rgb16.depth.rgbGreen = (BYTE)ddsd.ddpfPixelFormat.dwGBitMask;
   rgb16.Position.rgbGreen = shiftcount;
   rgb16.Amount.rgbGreen = (ddsd.ddpfPixelFormat.dwGBitMask == 0x1f) ? 3 : 2;
    
   // get Blue
   shiftcount = 0;
   while(!(ddsd.ddpfPixelFormat.dwBBitMask & 1))
   {
      ddsd.ddpfPixelFormat.dwBBitMask >>= 1;
      shiftcount++;
   }
   rgb16.depth.rgbBlue = (BYTE)ddsd.ddpfPixelFormat.dwBBitMask;
   rgb16.Position.rgbBlue = shiftcount;
   rgb16.Amount.rgbBlue = (ddsd.ddpfPixelFormat.dwBBitMask == 0x1f) ? 3 : 2;
    
   return TRUE;
}

// Now that we have our pixel format figured out, we can get to the fun stuff! Below are some macros for extracting the individual RGB components from a 16bit surface value, and a macro for creating a 16bit surface value from give red, green, blue values... 

#define RED(p) (p >> rgb16.Position.rgbRed) // Extracts Red Component
#define GREEN(p) ((p & rgb16.Mask.rgbGreen) >> rgb16.Position.rgbGreen) // Extracts Green Component
#define BLUE(p) (p & rgb16.Mask.rgbBlue) // Extracts Blue Component
#define RGB16(r, g, b) ((r << rgb16.Position.rgbRed) | (g << rgb16.Position.rgbGreen) | b) // Creates RGB Pixel Value
//So after locking a surface, you can simple do something like below to plot a pixel: 

 //  ddsd.lpSurface[20] = RGB16(255,0,0);



// you need this struct for the getrgb16 proc
// note:  you could even yank the bitmask variables out of this
// if you find you aren't using them in any functions of your own
typedef struct{
	DWORD dwRBitMask,
		dwGBitMask,	
		dwBBitMask; 
   RGBQUAD Position; 
  //  RGBQUAD is a data structure included with MSVC++5.0
} 
RGB16;
BOOL GetRGB16 ( LPDIRECTDRAWSURFACE Surface, RGB16 *rgb16){   
	DDSURFACEDESC   ddsd;    //get a surface description   
	ddsd.dwSize = sizeof( ddsd );    
	ddsd.dwFlags = DDSD_PIXELFORMAT;   
	if (Surface->GetSurfaceDesc ( &ddsd ) != DD_OK )	return FALSE; 
   char r=0, g=0, b=0;  
	for(int n = 1; n<65536; n<<="1)" { 
		// will increment r,g, or b for each 1 
		if(ddsd.ddpfPixelFormat.dwRBitMask & n) ++r; // in it's bitmask, which will give 
		if(ddsd.ddpfPixelFormat.dwGBitMask & n) ++g; // us the number of bits used for each 
		if(ddsd.ddpfPixelFormat.dwBBitMask & n) ++b; // color
 } // This is the part that assumes the format is RGB 
	rgb16->dwRBitMask = ddsd.ddpfPixelFormat.dwRBitMask;  
	rgb16->Position.rgbRed = g + b;  
	rgb16->dwGBitMask = ddsd.ddpfPixelFormat.dwGBitMask;  
	rgb16->Position.rgbGreen = b;   
	rgb16->dwBBitMask = ddsd.ddpfPixelFormat.dwBBitMask;  
	rgb16->Position.rgbBlue = 0;    return TRUE;
}//GetRBG16

Now that we have our format taken care of, here is myPlot_Pixel routine: 

// Must have the surface locked, pass the surface description 
// that gets returned from the lock call
void PlotRGB(int x, int y, int r, int g, int b, DDSURFACEDESC *ddsd, RGB16 *rgb16){
	WORD Pixel;   
	Pixel = (r << rgb16->Position.rgbRed) | (g << rgb16->Position.rgbGreen) | (b);	
	//  once again, assuming format is RGB 
	WORD *pixels = (WORD *)ddsd->lpSurface; 
	DWORD pitch = ddsd->dwWidth; 
	pixels[y*pitch + x] = Pixel;
}
*/