//////////////////////////////////////////////////////////////////////////////////
//
// Hippo Direct Draw Implementation
// (c) 2000
//
//////////////////////////////////////////////////////////////////////////////////


#include "stdafx.h"

#include "DDutils.h"
#include "HDDraw.h"

//////////////////////////////////////////////////////////////////////////////////
// Inicializace kompletni nulovani
//////////////////////////////////////////////////////////////////////////////////
void HDDraw::InitHDDraw()
{
	HRESULT   rval;

	// initialize member variables
	m_lpDD					= NULL;
	m_lpDDSGDI				= NULL;
	m_lpDDSBack				= NULL;

//	m_dcBack             = NULL;

	m_bFullScreen			= FALSE;
	m_GDIVisible         = TRUE;

	m_Rect.left = 0; m_Rect.top = 0;
	m_Rect.right = 0; m_Rect.bottom = 0;

	// tvorba direct draw object
	rval = DirectDrawCreate(NULL,(LPDIRECTDRAW *) &m_lpDD, NULL);
	if(rval != DD_OK) m_lpDD = NULL;
}


//////////////////////////////////////////////////////////////////////////////////
// Standartni konstruktor
//////////////////////////////////////////////////////////////////////////////////
HDDraw::HDDraw()
{
	InitHDDraw();
}

//////////////////////////////////////////////////////////////////////////////////
// ma sjteny efekt, jako normalni konstruktor a nasledne volani CreateFullScreen
//////////////////////////////////////////////////////////////////////////////////
HDDraw::HDDraw(HWND hWnd, DWORD Width, DWORD Height, DWORD BPP, DWORD Color)
{
	InitHDDraw();
	CreateFullScreen(hWnd, Width, Height, BPP, Color);
}

//////////////////////////////////////////////////////////////////////////////////
// destruktor - nici nenulove obekty
//////////////////////////////////////////////////////////////////////////////////
HDDraw::~HDDraw(void)
{

//	if(m_dcBack != NULL)
//		ReleaseBackDC();

	if( m_lpDDSBack != NULL ) {
		m_lpDDSBack->Release();
		m_lpDDSBack = NULL;
	}

	if( m_lpDDSGDI != NULL ) {
		m_lpDDSGDI->Release();
		m_lpDDSGDI = NULL;
	}
	

	if(m_lpDD != NULL) {
//		m_lpDD->RestoreDisplayMode();
		m_lpDD->Release();
	}

}

//////////////////////////////////////////////////////////////////////////////////
// prepne do fullscreenu a vytvori predni a zadni buffer - pouziva 16BPP (bytes per pixel)
//////////////////////////////////////////////////////////////////////////////////
BOOL HDDraw::CreateFullScreen(HWND hWnd, DWORD Width, DWORD Height, DWORD BPP, DWORD color)
{
	if(m_lpDD == NULL) return FALSE;

	HRESULT ddrval;

	DDSURFACEDESC ddsd;
	DDSCAPS ddscaps;

	// first release everything that has been allocated
	if( m_lpDDSBack != NULL ) {
		delete m_lpDDSBack;
		m_lpDDSBack = NULL;
	}

	if( m_lpDDSGDI != NULL ) {
		delete m_lpDDSGDI;
		m_lpDDSGDI = NULL;
	}


	m_bFullScreen = FALSE;

	// Get exclusive mode - spojeni s objektem okna
	ddrval = m_lpDD->SetCooperativeLevel(hWnd, DDSCL_EXCLUSIVE | DDSCL_FULLSCREEN | DDSCL_ALLOWREBOOT);
	if( ddrval != DD_OK ) return FALSE;

	// zmena rozliseni obrazovky
	ddrval = m_lpDD->SetDisplayMode( Width, Height, BPP);  // zmena rozliseni - 16-bitova barevna hloubka
   if( ddrval != DD_OK ) 
		return FALSE;

	// Create the primary surface with 1 back buffer - tvorba surfacu
	ddsd.dwSize = sizeof( ddsd );
	ddsd.dwFlags = DDSD_CAPS | DDSD_BACKBUFFERCOUNT;
	ddsd.ddsCaps.dwCaps = DDSCAPS_PRIMARYSURFACE | DDSCAPS_FLIP | DDSCAPS_COMPLEX;
	ddsd.dwBackBufferCount = 1; // pocet pouze pametovych obrazovek - na fillig - 1
	ddrval = m_lpDD->CreateSurface( &ddsd, &m_lpDDSGDI, NULL );
	if( ddrval != DD_OK ) return FALSE;

	FillGDI(color);


	// Get a pointer to the back buffer 
	ddscaps.dwCaps = DDSCAPS_BACKBUFFER;
   ddrval = m_lpDDSGDI->GetAttachedSurface(&ddscaps, &m_lpDDSBack);
	if( ddrval != DD_OK ) return FALSE;

	Fill(color);

	m_bFullScreen = TRUE;
	m_Rect.left = 0; m_Rect.top = 0;
	m_Rect.right = Width; m_Rect.bottom = Height;

	return TRUE;
}

//////////////////////////////////////////////////////////////////////////////////
// prohodi predni a zadni buffer
//////////////////////////////////////////////////////////////////////////////////
HRESULT HDDraw::Flip(BOOL VSync)
{
	HRESULT rval;


	if(m_bFullScreen)
	{
		if(m_FlipLock)
			rval = m_lpDDSGDI->BltFast(0,0,m_lpDDSBack,&m_Rect,DDBLTFAST_WAIT);
		else {
			if( VSync == TRUE )
				rval = m_lpDDSGDI->Flip(NULL, DDFLIP_WAIT);
			else
				rval = m_lpDDSGDI->Flip(NULL, DDFLIP_NOVSYNC);
		}

		if(rval == DDERR_SURFACELOST) Restore();
		if(rval == DD_OK) m_GDIVisible = !m_GDIVisible;
	}
	

	return rval;
}

//////////////////////////////////////////////////////////////////////////////////
// vyplni zadni buffer danou barvou
//////////////////////////////////////////////////////////////////////////////////
void HDDraw::Fill(DWORD FillColor)
{
	DDBLTFX ddBltFx;

	ddBltFx.dwSize = sizeof(DDBLTFX);
	ddBltFx.dwFillColor = FillColor;
	m_lpDDSBack->Blt(NULL, NULL, NULL, DDBLT_WAIT | DDBLT_COLORFILL, &ddBltFx);
}

//////////////////////////////////////////////////////////////////////////////////
// vyplni preni buffer danou barvou
//////////////////////////////////////////////////////////////////////////////////
void HDDraw::FillGDI(DWORD FillColor)
{
	DDBLTFX ddBltFx;

	ddBltFx.dwSize = sizeof(DDBLTFX);
	ddBltFx.dwFillColor = FillColor;
	m_lpDDSGDI->Blt(NULL, NULL, NULL, DDBLT_WAIT | DDBLT_COLORFILL, &ddBltFx);
}


//////////////////////////////////////////////////////////////////////////////////
// kdyz se surface ztrati
//////////////////////////////////////////////////////////////////////////////////
void HDDraw::Restore(void)
{
	if(m_bFullScreen) {
		// ve full screenu staci resornou t predn ibuffer, zadni se resotrne automaticky
		if(m_lpDDSGDI != NULL && m_lpDDSGDI->IsLost() != DD_OK)
			m_lpDDSGDI->Restore();
	}

}

//////////////////////////////////////////////////////////////////////////////////
// umoznuje pouzivat tridu i jako LPDIRECTDRAW
//////////////////////////////////////////////////////////////////////////////////
HDDraw::operator LPDIRECTDRAW() {
	return m_lpDD;
}

//////////////////////////////////////////////////////////////////////////////////
// Toto volejte pred zobrazovanim dialogu ve Full Screenu
//////////////////////////////////////////////////////////////////////////////////
HRESULT HDDraw::FlipToGDI()
{
	if(!m_GDIVisible) return Flip();
	return DD_OK;
}
//////////////////////////////////////////////////////////////////////////////////
// Pokud potrebujete psat, nebo kreslit na back surface
//////////////////////////////////////////////////////////////////////////////////
HDC HDDraw::GetBackDC()
{
	m_lpDDSBack->GetDC(&m_dcBack);
	return m_dcBack;
}

//////////////////////////////////////////////////////////////////////////////////
// kdyz dokreslite, tak zavoleste tohle - jinak nepude surface pouzit !!!
//////////////////////////////////////////////////////////////////////////////////
void HDDraw::ReleaseBackDC()
{
	m_lpDDSBack->ReleaseDC(m_dcBack);
//	m_dcBack = NULL;
}

//////////////////////////////////////////////////////////////////////////////////
// Tvori a vraci surface danych rozmeru
//////////////////////////////////////////////////////////////////////////////////
LPDIRECTDRAWSURFACE HDDraw::CreateSurface(DWORD dwWidth, DWORD dwHeight)
{
    DDSURFACEDESC       ddsd;
    LPDIRECTDRAWSURFACE pdds;

	 DDInitStruct(&ddsd);
    ddsd.dwFlags = DDSD_CAPS | DDSD_HEIGHT |DDSD_WIDTH;
    ddsd.ddsCaps.dwCaps = DDSCAPS_OFFSCREENPLAIN;
    ddsd.dwWidth = dwWidth;
    ddsd.dwHeight = dwHeight;

    if (m_lpDD->CreateSurface(&ddsd, &pdds, NULL) != DD_OK)
		return NULL;
	 // else
	 return pdds;
}
